#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include "cscope interface.h"
#include "SimpleScope.h"


void InitializeGUI (void);

int32 		panel_handle;
float64 	a_vert = 0.5, b_vert = 0.5;
float64		sig_gen_volts = 1.0, sig_gen_freq = 1000.0;
int32		channel = 1;
int32 		trig_channel = trig_chan_a;
float64		trig_volts = 0.0;
int32		single_acquire = 0;
int32		auto_acquire = 0;
float32 	a_waveform[max_samples], b_waveform[max_samples];
float64		x_delta, xzero;
int8 		pathname[MAX_PATHNAME_LEN];
int32		waiting_for_trigger = 0;	//0 means we have not started an acquire. 1 means we are waiting for the trigger
int32		waiting_for_replay = 0;		//0 means not waiting for a replay. 
int32		play_next_frame = 0;		//used to playback teh frames slowly


int32		vals_changed = 0;
uInt16 		trigger_action = 3;	//0 = auto, 1 = single, 3 = stop
uInt32 		digital_pattern =0x00010001;	//digital pattern used for digital triggering
int8		ip_adr_string[16] = "192.168.100.100";
uInt16  	Interface_Source;			//The interface source
uInt32 		Interface_address;			//the IP address for the interface
uInt16 		CAU_status;		//{c_runtime_closed, c_closed, c_open,  c_fault, c_fault_closed};
uInt16 		CAU_status_last;		//{c_runtime_closed, c_closed, c_open,  c_fault, c_fault_closed}; 
int32		Cscope_index;	//index into current cleverscope
int32		function_value;		//contains function number to send
int8		f_timer_active;		//true if teh timer is active
float64		start_time, stop_time;	//start and stop time for sequential capture
int32		num_samples_displayed;
int32		num_frames;				//num frames to capture
int32		num_samples;			//num samples to get
int32		current_frame = 0;

//---------------------------------------------------------------------------

int32 main (int argc, int8 *argv[])
{
int32		nErr = 0;
int8	message[100];

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */

    nErr =scope_init ();
    if (nErr>0)
    	{
    	sprintf (message, "There was a problem with the Cleverscope DLL, the error code = %d", nErr);
    	MessagePopup ("Problem with Cleverscope DLL", message);
    	return -1;
    	}

    panel_handle = LoadPanel (0, "SimpleScope.uir", PANEL);
    waiting_for_trigger = 0;
    vals_changed = 1;
    DisplayPanel (panel_handle);
    InitializeGUI();
    nErr = RunUserInterface ();
    CloseCVIRTE();
    return 0;
}
//---------------------------------------------------------------------------

int32  CVICALLBACK SingleAcquire(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_COMMIT)
    	{
        GetCtrlVal (panel_handle, PANEL_SINGLE, &single_acquire);
			//Single acquire will have toggled
        vals_changed = 1;
        waiting_for_trigger = 0;
        SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
		if (single_acquire)
			{
			trigger_action = acq_single;   //use triggered capture
			}
		else
			{
			trigger_action = acq_stop;	//stop triggering
			}

    	}
    return(0);

}
//---------------------------------------------------------------------------

int32 CVICALLBACK UpdateCallback (int32 panel, int32 control, int32 event,
        void *callbackData, int32 eventData1, int32 eventData2)
{
    switch (event)
    {
        case EVENT_TIMER_TICK:
        	SetCtrlAttribute (PANEL, PANEL_TIMER, ATTR_ENABLED, 0 );  //stop event timer

			//Update current CAU status

            CAU_status = get_CAU_status();
         
            if (CAU_status_last != CAU_status)
            	{
            	CAU_status_last = CAU_status; 
            	SetCtrlVal (panel_handle, PANEL_Cscope_Status, CAU_status);
            	SetCtrlVal (panel_handle, PANEL_Restart, 0);			//button press is completed
            	}
			//if the values have changed, update them in the CAU
             if (vals_changed==1)
            	{
            	update_values(a_vert, b_vert, start_time, stop_time, num_samples, num_frames, 
				sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action, digital_pattern);
				vals_changed = 0;
				}

		   	if (play_next_frame)
		   		{
		   		scope_get_frame(current_frame);
		   		SetCtrlVal (panel_handle, PANEL_CurrentFrame, current_frame);
		   		current_frame = current_frame +1;
		   		if (current_frame >= num_frames) 
		   			{
		   			play_next_frame = 0;
		   			}
	   			scope_read_waveform (a_waveform, b_waveform, &num_samples_displayed, &x_delta, &xzero);
	        	DeleteGraphPlot (panel_handle, PANEL_GRAPH, -1, 0);
				SetCtrlAttribute (panel_handle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS,VAL_LEFT_YAXIS);
				PlotWaveform (panel_handle, PANEL_GRAPH, a_waveform, num_samples_displayed,
					VAL_FLOAT, 1, 0, xzero, x_delta,
					VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
				SetCtrlAttribute (panel_handle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS,VAL_RIGHT_YAXIS);
				PlotWaveform (panel_handle, PANEL_GRAPH, b_waveform, num_samples_displayed,
					VAL_FLOAT, 1, 0, xzero, x_delta,
					VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_BLUE);
				}
			//If we are waiting for a trigger, and some samples have arrived get them and display them.
		    if (waiting_for_replay)
		    	{
		    	if (check_for_samples())
		    		{
		    		//here the replay has completed, sequence now in PC memory
		    		waiting_for_replay = 0;
		    		current_frame = 0;
		    		play_next_frame = 1;
		    		}
		    	}
	        if (waiting_for_trigger)
	           	{
	           	if (check_for_samples())
		       		{
		       		//here the sequence has completed. Now we need to replay the sequence so it comes into
		       		//PC memory. 
		       		SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 1);
					single_acquire = 0;
					SetCtrlVal (panel_handle, PANEL_SINGLE, single_acquire);
		       		waiting_for_trigger = 0;
		       		scope_replay();
		       		waiting_for_replay = 1;
		           	}
		        else
		           	{
		           	SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
					//no samples, turn off trigger LED
		           	}
		        }

			//If we are not waiting for samples, and teh user has selected either single or auto acquire
			//start an acquisition
	        if ((waiting_for_trigger == 0) && single_acquire )
	            {
 	            scope_acquire ();
	            waiting_for_trigger = 1;
	            }
			//finally, if something has changed, during acquisition, or while waiting for a trigger
			//update the CAU with the new values.
	        if (vals_changed == 1)
	           	{
            	update_values(a_vert, b_vert, start_time, stop_time, num_samples, num_frames, 
				sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action, digital_pattern);
				vals_changed = 0;
	           	scope_config();
	           	vals_changed = 0;		//updated if acquire did not update them
	           	}

	        SetCtrlAttribute (PANEL, PANEL_TIMER, ATTR_ENABLED, f_timer_active );  //turn off the event timer 
	        if (f_timer_active ==  0)
	        	{
				CAU_status = scope_close ();
	        	SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
        		QuitUserInterface (0);
        		}
            break;
    }

    return 0;
}

//---------------------------------------------------------------------------

int32  CVICALLBACK SetA_Vertical(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_A_VERTICAL, &a_vert);
        SetCtrlVal (panel_handle, PANEL_VA_READOUT, a_vert);
		SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL, -a_vert*v_divisions/2, a_vert*v_divisions/2);
        vals_changed = 1;
    }
    return (0);
}
//---------------------------------------------------------------------------

int32  CVICALLBACK SetB_Vertical(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_B_VERTICAL, &b_vert);
        SetCtrlVal (panel_handle, PANEL_VB_READOUT, b_vert);
		SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_RIGHT_YAXIS, VAL_MANUAL, -b_vert*v_divisions/2, b_vert*v_divisions/2);

        vals_changed = 1;
    }
    return (0);
}

//---------------------------------------------------------------------------

int32 CVICALLBACK SetChannel(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_COMMIT) {
        GetCtrlVal (panel_handle, PANEL_CHANNEL, &trig_channel);
        vals_changed = 1;
    }
    return(0);

}
//---------------------------------------------------------------------------

int32 CVICALLBACK SetDigPattern(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_COMMIT) {
        GetCtrlVal (panel_handle, PANEL_Dig_pattern, &digital_pattern);
        vals_changed = 1;
    }
    return(0);

}
//---------------------------------------------------------------------------

int32  CVICALLBACK Set_SigGen_Volts(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_SIGGENVOLTS, &sig_gen_volts);
        SetCtrlVal (panel_handle, PANEL_SIGGENVREADOUT, sig_gen_volts);
        vals_changed = 1;
    }
    return (0);
}
//---------------------------------------------------------------------------

int32  CVICALLBACK SetSigGenFreq(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_SIGGENFREQ, &sig_gen_freq);
        SetCtrlVal (panel_handle, PANEL_SIGGENFREADOUT, sig_gen_freq);
        vals_changed = 1;
    }
    return (0);
}
//---------------------------------------------------------------------------

int32  CVICALLBACK Set_Trigger_Volts(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_TRIGGER, &trig_volts);
        SetCtrlVal (panel_handle, PANEL_TRIGGERREADOUT, trig_volts);
        vals_changed = 1;
    }
    return (0);
}

//---------------------------------------------------------------------------
int32  CVICALLBACK Get_ip_address(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED)
    	{
        GetCtrlVal (panel_handle, PANEL_IP_address, ip_adr_string);
        Interface_address =  string_to_IP(ip_adr_string);
    	cscope_interface(Cscope_index, Interface_Source, Interface_address);
    	}
    return (0);
}

//---------------------------------------------------------------------------
int32  CVICALLBACK Get_Intf_source(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED)
    	{
        GetCtrlVal (panel_handle, PANEL_Intf_source, &Interface_Source);
    	cscope_interface(Cscope_index, Interface_Source, Interface_address);
    	}
    return (0);
}

//---------------------------------------------------------------------------
int32  CVICALLBACK Set_unit_index(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    if (event == EVENT_VAL_CHANGED)
    	{
        GetCtrlVal (panel_handle, PANEL_Cscope_unit_index , &Cscope_index);
    	cscope_interface(Cscope_index, Interface_Source, Interface_address);
    	}
    return (0);
}
//----------------------------------------------------------------------------

int32 CVICALLBACK RestartClick (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)  
		{
 		 cscope_interface(Cscope_index, Interface_Source, Interface_address);

		}
	return (0);
}
//---------------------------------------------------------------------------

int32 CVICALLBACK QuitButton(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{


    if (event == EVENT_COMMIT) 
    	{
		f_timer_active = 0; 
        }
    return(0);
}
//---------------------------------------------------------------------------

int32  CVICALLBACK Cmd_FunctionChange(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)

{
    if (event == EVENT_VAL_CHANGED)
    	{
        GetCtrlVal (panel_handle, PANEL_Cmd_Function , &function_value);
    	}
    return (0);
}
//---------------------------------------------------------------------------

int32 CVICALLBACK Cmd_getClick (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)  
		{
 		SetCtrlVal (panel_handle,PANEL_Cmd_value, get_function(function_value,0));
 		SetCtrlVal (panel_handle, PANEL_Cmd_get, 0); 
		}
	return (0);
}


int32 CVICALLBACK SaveToDisk(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    int8 directory[MAX_PATHNAME_LEN];
    FILE *file_handle;
    int32 i;

    if (event == EVENT_COMMIT) {
        GetProjectDir (directory);
        if (FileSelectPopup (directory, "*.wfm", "WFM Files",
                             "Save Waveform to Disk", VAL_SAVE_BUTTON, 1,
                             1, 1, 0, pathname) > 0) {
        file_handle = fopen (pathname, "w+");
        fprintf (file_handle, "OSCILLOSCOPE DEMO DATA FILE \n\n");
        fprintf(file_handle, "Channel Acquired: CH%d\n", channel);
        fprintf(file_handle, "Date %s   Time: %s \n", DateStr(), TimeStr());
        fprintf (file_handle, "----------------------------------\n");
        for (i=0;i<100;i++)
            fprintf (file_handle, "%f\n", a_waveform[i]);
        fclose (file_handle);
        }
    }
    return (0);
}

//---------------------------------------------------------------------------

int32 CVICALLBACK ViewDataPoints(int32 panel, int32 control, int32 event, void *callbackData, int32 eventData1, int32 eventData2)
{
    int8 filestring[100];

    if (event == EVENT_COMMIT) {
        sprintf (filestring, "notepad.exe %s", pathname);
        LaunchExecutable (filestring);
        }
    return (0);
}
//---------------------------------------------------------------------------

void InitializeGUI ()
{
	single_acquire = 0;
	auto_acquire = 0;
	waiting_for_trigger = 0;
	vals_changed = 1;
	trigger_action = acq_stop;   //stop triggering
	Interface_Source =if_usb ;			//The interface source
	Cscope_index = 0;	//index into current cleverscope
	start_time = -0.005;
	stop_time = 0.005;
	num_frames = 10;
	num_samples = 1000;

	SetCtrlVal (panel_handle, PANEL_Cscope_Status, c_runtime_closed);
    SetCtrlVal (panel_handle, PANEL_VA_READOUT, a_vert);
    SetCtrlVal (panel_handle, PANEL_VB_READOUT, b_vert);
    SetCtrlVal (panel_handle, PANEL_TRIGGERREADOUT, trig_volts);
    SetCtrlVal (panel_handle, PANEL_SIGGENVREADOUT, sig_gen_volts);
    SetCtrlVal (panel_handle, PANEL_SIGGENFREADOUT, sig_gen_freq);
    SetCtrlVal (panel_handle, PANEL_CHANNEL, trig_channel);
    SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);

    SetCtrlVal (panel_handle, PANEL_A_VERTICAL, a_vert);
    SetCtrlVal (panel_handle, PANEL_B_VERTICAL, b_vert);
    SetCtrlVal (panel_handle, PANEL_TRIGGER, trig_volts);
    SetCtrlVal (panel_handle, PANEL_SIGGENVOLTS, sig_gen_volts);
    SetCtrlVal (panel_handle, PANEL_Cmd_value, 0);
    SetCtrlVal (panel_handle, PANEL_StartTime, start_time);
    SetCtrlVal (panel_handle, PANEL_StopTime, stop_time);
    SetCtrlVal (panel_handle, PANEL_NumFrames, num_frames);
    SetCtrlVal (panel_handle, PANEL_NumSamples, num_samples);
    
	SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_BOTTOM_XAXIS, VAL_MANUAL, start_time, stop_time);    
	SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL, -a_vert*v_divisions/2, a_vert*v_divisions/2);
	SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_RIGHT_YAXIS, VAL_MANUAL, -b_vert*v_divisions/2, b_vert*v_divisions/2);
	f_timer_active = 1;
	SetCtrlAttribute (PANEL, PANEL_TIMER, ATTR_ENABLED, 1 );  //start event timer

}
//---------------------------------------------------------------------------
int CVICALLBACK PanelResponse (int handle, int event, void *callbackdata, int eventdata1, int eventdata2)

{

switch (event) {

case EVENT_CLOSE :
f_timer_active = 0;
	//Code that responds to the panel being being closed
break;


}

return(0);

}
//-------------------------------------------------------------------------

int CVICALLBACK SetStartTime (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panel_handle, PANEL_StartTime , &start_time);
			SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_BOTTOM_XAXIS, VAL_MANUAL, start_time, stop_time);

			break;
		}
	return 0;
}

int CVICALLBACK SetStopTime (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panel_handle, PANEL_StopTime , &stop_time);
			SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_BOTTOM_XAXIS, VAL_MANUAL, start_time, stop_time);
			break;
		}
	return 0;
}

int CVICALLBACK SetNumFrames (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panel_handle, PANEL_NumFrames , &num_frames);
			break;
		}
	return 0;
}

int CVICALLBACK SetNumSamples (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panel_handle, PANEL_NumSamples , &num_samples); 
			break;
		}
	return 0;
}

//------------------------------------------------------------------------------
